C  "HKLGEN" GENERATES (H,K,L)S AND CALCULATES POWDER PATTERN MULTIPLICITIES
C    FOR A GIVEN LAUE GROUP AND LATTICE TYPE
C  THE OUTPUT IS SUITABLE FOR USE IN THE RIETVELD PROFILE REFINEMENT PROGRAM
C  J.B.FORSYTH, NEUTRON BEAM UNIT, RUTHERFORD LABORATORY,HARWELL, ENGLAND
C  MODIFIED AND MOUNTED AT THE FRM BY W.NAEGELE 2 APR 1976
C
C                              DATA REQUIRED
C
C  1ST CARD (10A8) TITLE
C  2ND CARD (6F8) A,B,C,ALPHA,BETA,GAMMA  (CELL DIMENSIONS AND ANGLES)
C  3RD CARD (2F8,3I8,4X,A4) WAVELENGTH, MAX 2THETA, LAUE GROUP, LATTICE TYPE,
C                           1 FOR CARD OUTPUT
C
C  CODE IS    LAUE GROUP                    LATTICE TYPE
C             1 TRICLINIC                   1 PRIMITIVE
C             2 MONOCLINIC (B-AXIS UNIQUE)  2 A-FACE CENTRED
C             3 ORTHORHOMBIC                3 B-FACE CENTRED
C             4 TETRAGONAL 4/M              4 C-FACE CENTRED
C             5 TETRAGONAL 4/MMM            5 ALL FACES CENTRED
C             6 TRIGONAL BAR 3              6 BODY CENTRED
C             7 TRIGONAL BAR 3/M            7 RHOMBOHEDRAL (HEX. INDICES)
C             8 HEXAGONAL 6/M
C             9 HEXAGONAL 6/MMM
C            10 CUBIC M3
C            11 CUBIC M3M
C            12 MONOCLINIC (C-AXIS UNIQUE)
C
C  4TH CARD (I8)
C  5TH CARD (9F8)
C
C                                EXAMPLE OF DATA
C
C TETRAIODOETHYLENE      5.10.73
C 15.142   4.338  12.899  90.    108.35   90.
C  1.5087 160.0         2       1
C
C
C      PROGRAM HKL(HKLLIS,LPHKL,INPUT,OUTPUT,TAPE5=INPUT,TAPE6=OUTPUT,
C     1TAPE1=HKLLIS,TAPE7=LPHKL)
      INTEGER H,HMIN,HMAX
      INTEGER   HH,KK,LL,M
      REAL LUVW
      logical ERR
      DIMENSION CARD(10),HH(2000),KK(2000),LL(2000),DD(2000),M(2000),
     1MARK(2000),IND(2000),U(3),V(3),W(3),LUVW(3),SO2(3)
C
C           READ INPUT DATA
C
       OPEN(1,FILE="hkllis")
       OPEN(7,FILE="lphkl")
      ERR=.FALSE.
        WRITE(6,2010)
2010    FORMAT(1X,"GIB TEXT",/)
        READ(5,1000)(CARD(I),I=1,10)
1000    FORMAT(10A8)
        WRITE(6,2020)
2020    FORMAT(1X,"GIB ZELLDIMENSIONEN UND WINKEL",/)
        READ(5,*)A,B,C,ALPHA,BETA,GAM
        WRITE(6,2040)
2040    FORMAT(1X,"GIB WELLENLAENGE UND MAX. 2-THETA",/)
        READ(5,*)WAVEL,TTMAX
        WRITE(6,2060)
2060    FORMAT(" GIB LAUE-GRUPPE UND GITTERTYP",/)
        WRITE(6,1002)
1002    FORMAT(/,1X,"LAUE-GRUPPE",25X,"GITTERTYP",/
     11X,"1 TRIKLIN",27X,"1 PRIMITIV",/
     11X,"2 MONOKLIN (B-ACHSE BEVORZUGT)",6X,"2 A-ZENTRIERT",/
     11X,"3 ORTHORHOMBISCH",20X,"3 B-ZENTRIERT",/
     11X,"4 TETRAGONAL 4/M",20X,"4 C-ZENTRIERT",/
     11X,"5 TETRAGONAL 4/MMM",18X,"5 F-ZENTRIERT",/
     11X,"6 TRIGONAL 3 QUER",19X,"6 I-ZENTRIERT",/
     11X,"7 TRIGONAL 3 QUER/M",17X,"7 RHOMBOEDRISCH (HEX. INDIZES)",/
     11X,"8 HEXAGONAL 6/M",/
     11X,"9 HEXAGONAL 6/MMM",/
     11X,"10 KUBISCH M3",/
     11X,"11 KUBISCH M3M",/
     11X,"12 MONOKLIN (C-ACHSE BEVORZUGT)",//)
        READ(5,*)NLAUE,NLAT
        WRITE(6,2080)
2080    FORMAT(1X,"GIB IOP UND ICODE",/," IOP=1 FUER HKL-FILE",/
     1"0 NUR LINE-PRINTER OUTPUT",/
     1" ICODE=1 NUR KERNREFLEXE",/
     1"2 NUR MAGNETISCHE REFLEXE",/
     1"3 BEIDES",/)
        READ(5,*)IOP,ICODE
C
        MM=0
        IF (IOP.EQ.0) GO TO 52
      NF=1
C           WRITE(7,INPUT) DATA
C
   52    WRITE(7,1010)(CARD(I),I=1,10)
 1010    FORMAT (1H1,/,1H0,10A8)
         WRITE(7,1011)A,B,C,ALPHA,BETA,GAM
 1011 FORMAT (1H0,10X,27HDIMENSIONS OF THE UNIT CELL,
     13X,2HA=,F10.4,1X,9HANGSTROMS/
     141X,2HB=,F10.4/
     141X,2HC=,F10.4/
     137X,6HALPHA=,F10.4,8H DEGREES/
     138X,5HBETA=,F10.4/
     137X,6HGAMMA=,F10.4//)
         DMIN=WAVEL/(2.*SIN(0.00872665*TTMAX))
         WRITE(7,1012)WAVEL,TTMAX,NLAUE,NLAT
 1012    FORMAT (22X,20HNEUTRON WAVELENGTH =,F10.4,10H ANGSTROMS,//
     135X,13H MAX 2THETA =,F10.4,8H DEGREES///
     113X,"LAUE TYPE",I4,12X,"1 TRICLINIC",14X,"LATTICE TYPE",I4,10X,
     1"1 PRIMITIVE"/
     138X,"2 MONOCLINIC (B-AXIS UNIQUE)",T90,"2 A FACE CENTRED"/
     138X,"3 ORTHORHOMBIC",T90,"3 B FACE CENTRED"/
     138X,"4 TETRAGONAL 4/M",T90,"4 C FACE CENTRED"/
     138X,"5 TETRAGONAL 4/MMM",T90,"5 ALL FACE CENTRED"/
     138X,"6 TRIGONAL BAR 3",T90,"6 BODY CENTRED"/
     138X,"7 TRIGONAL BAR 3/M",T90,"7 RHOMBOHEDRAL (HEX. INDICES)"/
     138X,"8 HEXAGONAL 6/M"/
     138X,"9 HEXAGONAL 6/MMM"/
     137X,"10 CUBIC M3"/
     137X,"11 CUBIC M3M"/
     137X,"12 MONOCLINIC (C-AXIS UNIQUE)"///)
C
C     CHECKS FOR ALLOWED VALUES OF NLAUE AND NLAT
C
      IF(NLAT.LE.0.OR.NLAT.GT.7) GO TO 34
      IF(NLAUE.LE.0.OR.NLAUE.GT.12) GO TO 35
      GO TO 36
   34 WRITE(7,1022)
 1022 FORMAT("1NON ALLOWED VALUE FOR NLAT REQUESTED")
      GO TO 998
  35  WRITE(7,1023)
 1023 FORMAT("1NON ALLOWED VALUE FOR NLAUE REQUESTED")
      GO TO 998
 36   IF(NLAT.NE.7) GO TO 33
      IF(NLAUE-6.EQ.0.OR.NLAUE-7.EQ.0) GO TO 33
   32 WRITE(7,1021)
 1021 FORMAT("1RHOMBOHEDRAL CELL IN HEXAGONAL INDICES REQUESTED FOR"/"
     1TRIGONAL LAUE TYPE")
  33  RAD=0.0174533
      ALPHA=ALPHA*RAD
      BETA=BETA*RAD
      GAM=GAM*RAD
      S=(ALPHA+BETA+GAM)/2
       VOL=2*A*B*C*SQRT(SIN(S)*SIN(S-ALPHA)*SIN(S-BETA)*SIN(S-GAM))
C
         AA=B*C*SIN(ALPHA)/VOL
         BB=C*A*SIN(BETA)/VOL
         CC=A*B*SIN(GAM   )/VOL
C
         CALPHA=(COS(BETA)*COS(GAM  )-COS(ALPHA))/(SIN(BETA)*          S
     1IN(GAM  ))
         CBETA=(COS(GAM  )*COS(ALPHA)-COS(BETA))/(SIN(GAM  )           *
     1SIN(ALPHA))
         CGAMMA=(COS(ALPHA)*COS(BETA)-COS(GAM  ))/(SIN(ALPHA)*         S
     1IN(BETA))
C
         WRITE(7,1013)AA,BB,CC,CALPHA,CBETA,CGAMMA
 1013    FORMAT (9X,26HRECIPROCAL CELL PARAMETERS/
     140X,3HA*=,F10.4/
     140X,3HB*=,F10.4/
     140X,3HC*=,F10.4/
     131X,12HCOS(ALPHA*)=,F10.4/
     131X,12HCOS(BETA*) =,F10.4/
     131X,12HCOS(GAMMA*)=,F10.4///)
C
C           CALCULATE L(UVW)
C
      IF (MM) 60,60,54
   54 DO 50 I=1,MM
         UA=U(I)*A
         VB=V(I)*B
         WC=W(I)*C
         LUVW(I)=SQRT(UA*UA+VB*VB+WC*WC+2.0*VB*WC*COS(ALPHA)+
     12.0*UA*VB*COS(GAM  )+2.0*UA*WC*COS(BETA))
   50 CONTINUE
   60 CONTINUE
C
C           SET UP LIMITS OF H,K,L SCANS AND INITIAL VALUES FOR EACH CYCLE
C
   80    IC=1
C
C NOW SET UP THE MAXIMUM AND MINIMUM VALUES FOR H,K,L
C   AND INITIAL VALUES FOR THE SCAN THROUGH H,K,L
C
      HMAX=INT(A/DMIN)
      KMAX=INT(B/DMIN)
      LMAX=INT(C/DMIN)
      HMIN=0
      KMIN=0
      L=0
      IF(NLAUE.EQ.12) GO TO 11
      IF(NLAUE-2)10,11,12
   12 IF(NLAT.EQ.7) GO TO 13
      IF(NLAUE.EQ.6.OR.NLAUE.EQ.7) GO TO 11
      GO TO 13
   10 KMIN=-KMAX
   11 HMIN=-HMAX
   13 IF(NLAUE.EQ.10.OR.NLAUE.EQ.11) GO TO 62
      K=KMIN
        GO TO 14
62      K=L
   14 IF(NLAUE.EQ.12) GO TO 141
      IF(NLAUE.EQ.4.OR.NLAUE.EQ.5.OR.NLAUE.GE.8.OR.NLAT.EQ.7)
     1GO TO 61
  141 H=HMIN
      GO TO 95
   61 H=K
C
C     NOW REMOVE UNWANTED HKL"S
C
   95 CALL CHECK(H,K,L,NLAUE,NLAT,MULT,ERR)
      IF (ERR) GOTO 22
C           CALCULATE D*
   19    AH=AA*H
         BK=BB*K
         CL=CC*L
         Q=SQRT(AH*AH+BK*BK+CL*CL+2*AH*BK*CGAMMA+2*BK*CL*CALPHA
     1+2*CL*AH*CBETA)
         D=1./Q
C
      IF (D-DMIN) 22,21,21
C
C           ADD POINT TO ARRAY
   21    HH(IC)=H
         KK(IC)=K
         LL(IC)=L
         DD(IC)=D
         M(IC)=MULT
         IC=IC+1
C
      IF (IC-999) 22,22,73
   73    WRITE(7,1020)H,K,L
 1020    FORMAT (50H0 ARRAY SIZE LIMITED , SCAN THRU H,K,L STOPPED AT ,
     1   3I5/)
      GO TO 96
C    NOW STEP ON H,K,L   H FASTER THAN K, FASTER THAN L
C
   22 H=H+1
      IF(HMAX-H)16,95,95
   16 K=K+1
      IF(KMAX-K)17,14,14
   17 L=L+1
      IF(LMAX-L)96,13,13
   96    CONTINUE
         IC=IC-1
C
C           WRITE(7,OUTPUT) TITLE
      IF (MM) 56,56,57
   57    WRITE(7,1015)(U(II),V(II),W(II),II=1,MM)
 1015    FORMAT(/ 4X,1HH,4X,1HK,4X,1HL,6X,2HD ,4X,10HSIN(THETA),3X,
     17H2*THETA,5X,1HJ,3X,13HJ(1+COSTH**2),7X,1HJ,15X,
     121HSIN(OMEGA)**2,S ON TO/
     127X,9H---------,20X,12H------------,5X,12H------------/
     130X,5HLAMDA,21X,12HSINTH*SIN2TH,5X,12HSINTH*SIN2TH,4X,
     13(3F4.1,1X)//)
      GO TO 58
   56    WRITE(7,1019)
 1019    FORMAT(/ 4X,1HH,4X,1HK,4X,1HL,6X,2HD ,4X,10HSIN(THETA),3X,
     17H2*THETA,5X,1HJ,3X,13HJ(1+COSTH**2),7X,1HJ/
     127X,9H---------,20X,12H------------,5X,12H------------/
     130X,5HLAMDA,21X,12HSINTH*SIN2TH,5X,12HSINTH*SIN2TH//)
C
C           SORT ARRAY IN DESCENDING D
C
   58 CALL KB04A (DD,IC,IC,IND,MARK)
C
      IF(IOP.NE.0) WRITE(NF,1029) IC
 1029 FORMAT(1X,I7/7X,1H1,7X,1H1,7X,1H1)
      DO 42 I=1,IC
      J=IND(I)
C           CALCULATE OUTPUT VARIABLES
         X=1./(2.*DD(I))
         STHE=X*WAVEL
         CTHE=SQRT(ABS(1.0-STHE**2))
      ATHE2=2.0*ASIN(STHE)/RAD
      RS2TH=1.0/(2.0*STHE*STHE*CTHE)
         XCTH2=(1.0+CTHE*CTHE)*M(J)*RS2TH
         Y=M(J)*RS2TH
      IF (MM) 59,59,55
C
C           CALCULATE PROJECTION ALONG U,V,W ,S
C
   55 DO 51 II=1,MM
         CO=(U(II)*HH(J)+V(II)*KK(J)+W(II)*LL(J))*DD(I)/LUVW(II)
         SO2(II)=1.0-CO*CO
   51 CONTINUE
C
C           WRITE(7,OUT PUT
         WRITE(7,1014)HH(J),KK(J),LL(J),DD(I),X,ATHE2,M(J),XCTH2,Y,
     1(SO2(II),II=1,MM)
 1014    FORMAT (3I5,2F10.4,F12.4,I6,F11.3,7X,F10.3,3F16.5)
      GO TO 41
C
   59    WRITE(7,1018)HH(J),KK(J),LL(J),DD(I),X,ATHE2,M(J),XCTH2,Y
 1018    FORMAT (3I5,2F10.4,F12.4,I6,F11.3,7X,F10.3)
C
C     NOW PUNCH CARDS FOR RIETVELD PROGRAM IF REQUIRED
C
   41 IF(IOP.EQ.0) GO TO 43
      WRITE(NF,1024) ICODE,HH(J),KK(J),LL(J),M(J)
1024    FORMAT(5I8)
      GO TO 43
C
C     NOW ADD ANY ADDITIONAL COINCIDENCES
C
  43  IF(NLAUE.EQ.4.OR.NLAUE.EQ.6.OR.NLAUE.EQ.7.OR.NLAUE.EQ.8
     1.OR.NLAUE.EQ.10) CALL COINC(HH(J),KK(J),LL(J),M(J),
     2ICODE,NLAUE,NLAT,IOP,NF)
   42 CONTINUE
 998  CONTINUE
      END
      SUBROUTINE CHECK(H,K,L,NLAUE,NLAT,M,ERR)
      INTEGER H
      logical ERR
C
C           THIS SUBROUTINE TESTS THAT H,K,L OBEY CERTAIN RULES , THE
C           RULES DEPENDING ON THE LATTICE AND LAUE TYPES
C           IF THE RULES ARE OBEYED, RETURN WITH M SET TO MULTIPLICITY
C           IF THE RULES ARE NOT OBEYED, RETURN MR
C
C
C
C      FIRST REMOVE LATTICE ABSENCES
C
      ERR=.FALSE.
      GO TO (101,102,103,104,105,106,107) NLAT
C     A FACE CENTRED
C
  102 IF(MOD(K+L,2).EQ.0) GO TO 101
      ERR=.true.
      RETURN
C
C    B FACE CENTRED
C
  103 IF(MOD(H+L,2).EQ.0) GO TO 101
      ERR=.TRUE.
      RETURN
C
C
C        C FACE CENTRED
C
  104 IF(MOD(H+K,2).EQ.0) GO TO 101
      ERR=.TRUE.
      RETURN
C
C     ALL FACE CENTRED
C
  105 IF(MOD(H+K,2).EQ.0.AND.MOD(K+L,2).EQ.0.AND.MOD(H+L,2).EQ.0)
     C GO TO 101
      ERR=.TRUE.
      RETURN
C
C    BODY CENTRED
C
  106 IF(MOD(H+K+L,2).EQ.0) GO TO 101
      ERR=.TRUE.
      RETURN
C
C     RHOMBOHEDRAL CELL IN HEXAGONAL INDICES
C
  107 IF(MOD(-H+K+L,3).EQ.0) GO TO 101
      ERR=.TRUE.
      RETURN
C
C PRIMITIVE LATTICES, NO FURTHER CHECKS
C
C
  101 CONTINUE
C
C   NOW EXCLUDE REFLECTION (000)
C
      IF(H.EQ.0.AND.K.EQ.0.AND.L.EQ.0) GO TO 14
C
C     NOW REMOVE UNWANTED INDICES ACCORDING TO LAUE WRITE(6,AND GET
C     MULTIPLICITY.
C
      GO TO (1,2,3,4,5,6,7,8,9,10,11,112),NLAUE
C
C     TRICLINIC
C
    1 IF(L.EQ.0.AND.H.LT.0) GO TO 14
      IF(L.EQ.0.AND.H.EQ.0.AND.K.LT.0) GO TO 14
      GO TO 20
C
C     MONOCLINIC (C-AXIS UNIQUE)
C
  112 IF(K.EQ.0.AND.H.LT.0) GO TO 14
      IF(L.EQ.0) GO TO 20
      IF(H.EQ.0.AND.K.EQ.0) GO TO 20
      GO TO 40
C
C     MONOCLINIC (B-AXIS UNIQUE)
C
    2 IF(L.EQ.0.AND.H.LT.0) GO TO 14
      IF(K.EQ.0) GO TO 20
      IF(H.EQ.0.AND.L.EQ.0) GO TO 20
      GO TO 40
C
C     ORTHORHOMBIC
C
    3 IF(H.EQ.0.AND.K.EQ.0) GO TO 20
      IF(H.EQ.0.AND.L.EQ.0) GO TO 20
      IF(K.EQ.0.AND.L.EQ.0) GO TO 20
      IF(H.EQ.0.OR.K.EQ.0.OR.L.EQ.0) GO TO 40
      GO TO 80
C
C     TETRAGONAL  4/M
C
    4 IF(H.EQ.0.AND.K.EQ.0) GO TO 20
      IF(L.EQ.0) GO TO 40
      GO TO 80
C
C      TETRAGONAL  4/MMM
C
    5 IF(H.EQ.0.AND.K.EQ.0) GO TO 20
      IF(L.EQ.0.AND.K.EQ.0) GO TO 40
      IF(H-K.EQ.0.AND.L.EQ.0) GO TO 40
      IF(L.EQ.0.OR.K.EQ.0.OR.H.EQ.0) GO TO 80
      IF(H-K.EQ.0) GO TO 80
      GO TO 160
C
C      TRIGONAL  BAR 3
C
    6 IF(NLAT.EQ.7) GO TO 12
      IF(K.EQ.0.AND.L.EQ.0.AND.H.LT.0)GO TO 14
      IF(L-K)14,17,17
   17 IF(K-H)14,18,18
   18 IF(K.NE.0) GO TO 19
C
C     IF K=0, MOD(H) MUST BE LESS THAN OR EQUAL TO L
C
      IF(L-IABS(H))14,19,19
   19 IF(H-K.EQ.0.AND.K-L.EQ.0.AND.H-L.EQ.0) GO TO 20
      GO TO 60
C
C     TRIGONAL  BAR 3/M
C
    7 IF(NLAT.EQ.7) GO TO 13
      IF(K.EQ.0.AND.L.EQ.0.AND.H.LT.0)GO TO 14
      IF(L-K)14,200,200
  200 IF(K-H)14,201,201
  201 IF(K.NE.0) GO TO 202
C
C     IF K=0, MOD(H) MUST BE LESS THAN OR EQUAL TO L
C
      IF(L-IABS(H))14,202,202
  202 IF(H-K.EQ.0.AND.K-L.EQ.0.AND.H-L.EQ.0) GO TO 20
      IF(H+K+L.NE.0) GO TO 15
      IF(H.EQ.0.OR.K.EQ.0.OR.L.EQ.0) GO TO 60
      IF(H-K.EQ.0.OR.K-L.EQ.0.OR.L-H.EQ.0) GO TO 60
      GO TO 120
   15 IF(H-K.EQ.0.OR.K-L.EQ.0.OR.L-H.EQ.0) GO TO 60
      GO TO 120
C
C     TRIGONAL  BAR 3 IN HEXAGONAL INDICES
C
   12 IF(H.EQ.0.AND.K.EQ.0) GO TO 20
      GO TO 60
C
C     TRIGONAL  BAR 3/M  IN HEXAGONAL INDICES
C
   13 IF(H.NE.0.AND.K.EQ.0) GO TO 60
      IF(H-K.EQ.0.AND.L.EQ.0) GO TO 60
      IF(H.EQ.0.AND.K.EQ.0) GO TO 20
      GO TO 120
C
C     HEXAGONAL  6/M
C
    8 IF(H.EQ.0.AND.K.EQ.0) GO TO 20
      IF(L.EQ.0) GO TO 60
      GO TO 120
C
C     HEXAGONAL  6/MMM
C
    9 IF(H.EQ.0.AND.K.EQ.0) GO TO 20
      IF(H-K.EQ.0.AND.L.EQ.0) GOTO 60
      IF(K.EQ.0.AND.L.EQ.0) GOTO 60
      IF(L.EQ.0) GO TO 120
      IF(K.EQ.0.OR.H-K.EQ.0) GO TO 120
      GO TO 240
C
C     CUBIC  M3
C
   10 IF(K.EQ.0.AND.L.EQ.0) GO TO 60
      IF(H-K.EQ.0.AND.K-L.EQ.0.AND.H-L.EQ.0) GO TO 80
      IF(L.EQ.0) GO TO 120
      GO TO 240
C
C     CUBIC M3M
C
   11 IF(K.EQ.0.AND.L.EQ.0) GO TO 60
      IF(H-K.EQ.0.AND.H-L.EQ.0.AND.L-H.EQ.0) GO TO 80
      IF(H-K.EQ.0.AND.L.EQ.0) GO TO 120
      IF(L.EQ.0) GO TO 240
      IF(H-K.EQ.0.OR.K-L.EQ.0) GO TO 240
      GO TO 480
   20 M=2
      RETURN
   40 M=4
      RETURN
   60 M=6
      RETURN
   80 M=8
      RETURN
  120 M=12
      RETURN
  160 M=16
      RETURN
  240 M=24
      RETURN
  480 M=48
      RETURN
C
C  NOT ALLOWED
C
   14 ERR=.TRUE.
      RETURN
      END
      SUBROUTINE COINC(H,K,L,M,ICODE,NLAUE,NLAT,IOP,NF)
      INTEGER   H,K,L
      IF(NLAUE.EQ.6) GO TO 3
      IF(NLAUE.EQ.7) GO TO 4
      IF(H-K.NE.0) GO TO 2
      IF(H-K.NE.0.AND.L.NE.0.AND.K.NE.0) GO TO 2
      RETURN
    2 WRITE(7,100) K,H,L
  100 FORMAT(3I5)
      IF(IOP.NE.0) WRITE(NF,1024) ICODE,K,H,L,M
1024    FORMAT(5I8)
      RETURN
    3 IF(NLAT.EQ.7) GO TO 5
      RETURN
    4 IF(NLAT.EQ.7) GO TO 6
      RETURN
    5 IF(H-K.NE.0.AND.K.NE.0.AND.L.NE.0) GO TO 7
      IF(H-K.EQ.0.AND.K.NE.0.AND.L.NE.0) GO TO 8
      IF(H.NE.0.AND.K.EQ.0.AND.L.NE.0) GO TO 2
      IF(H-K.EQ.0.AND.K.NE.0.AND.L.EQ.0) GO TO 2
      RETURN
    7 WRITE(7,100) K,H,L
      IF(IOP.NE.0) WRITE(NF,1024) ICODE,K,H,L,M
      H=-H
      K=-K
      WRITE(7,100) H,K,L,K,H,L
      IF(IOP.NE.0) WRITE(NF,1024) ICODE,H,K,L,M,ICODE,K,H,L,M
      RETURN
    8 H=-H
      K=-K
      WRITE(7,100) H,K,L
      IF(IOP.NE.0) WRITE(NF,1024) ICODE,H,K,L,M
      RETURN
    6 IF(H-K.NE.0.AND.K.NE.0.AND.L.NE.0) GO TO 2
      IF(H.NE.0.AND.K.EQ.0.AND.L.EQ.0) GO TO 8
      RETURN
      END
      SUBROUTINE KB04A (COUNT,N,IA,IND,MARK)                                   1
C###### 04/08/72 LAST LIBRARY UPDATE                                           2
      DIMENSION COUNT(1),MARK(1),IND(1)                                        3
      DO 10 I=1,N                                                              4
   10 IND(I)=I                                                                 5
      IF(N.LT.2) RETURN                                                        6
      M=12                                                                     7
   20 M=2*M                                                                    8
      NP1=N+1                                                                  9
      MARK(1)=80000*NP1+1                                                     10
      NALM=2                                                                  11
      IM=1                                                                    12
   30 IF(MARK(IM))90,90,40                                                    13
   40 M3=MARK(IM)/NP1                                                         14
      IS=MARK(IM)-NP1*M3                                                      15
      IF(M3-80000)60,50,60                                                    16
   50 IF=N                                                                    17
      GO TO 70                                                                18
   60 IF=IABS(MARK(M3))-(IABS(MARK(M3))/NP1)*NP1-1                            19
   70 IF(NALM-IA)80,80,20                                                     20
   80 CALL KB04XA (IS,IF,IP,COUNT,IND)                                        21
      CALL KB04YA (IS,IF,IP,IM,NALM,COUNT,M,MARK,N,IND)                       22
      GO TO 30                                                                23
   90 IF(IM-NALM+1)100,110,110                                                24
  100 IM=IM+1                                                                 25
      GO TO 30                                                                26
  110 IM=1                                                                    27
  120 M3=-MARK(IM)/NP1                                                        28
      IS=-MARK(IM)-NP1*M3                                                     29
      IF(M3-80000)130,140,130                                                 30
  130 IF=-MARK(M3)+(MARK(M3)/NP1)*NP1-1                                       31
      GO TO 150                                                               32
  140 IF=N                                                                    33
  150 IF(IF-IS)160,170,160                                                    34
  160 CALL KB04ZA(COUNT,IF-IS+1,IS,IND)                                       35
  170 IM=M3                                                                   36
      IF(IM-80000)120,180,120                                                 37
  180 RETURN                                                                  38
      END
      SUBROUTINE KB04XA (IS,IF,IP,COUNT,IND)                                  40
      DIMENSION COUNT(1),IND(1)                                               41
      IY=(IS+IF)/2                                                            42
      X=COUNT(IY)                                                             43
      INT=IND(IY)                                                             44
      JJ=1                                                                    45
      IFK=IF+1                                                                46
      DO 80 I=IS,IF                                                           47
      J=JJ                                                                    48
      IF(I-IFK)10,90,90                                                       49
   10 IF (X-COUNT(I)) 80,20,30                                                50
   20 IF (INT-IND(I)) 30,80,80                                                51
   30 DO 70 K=J,IF                                                            52
      IFK=IF+1-K                                                              53
      IF(I-IFK)40,90,90                                                       54
   40 JJ=JJ+1                                                                 55
      IF (COUNT(IFK)-X) 70,50,60                                              56
   50 IF (IND(IFK)-INT) 60,60,70                                              57
   60 AV=COUNT(IFK)                                                           58
      COUNT(IFK)=COUNT(I)                                                     59
      COUNT(I)=AV                                                             60
      IV=IND(IFK)                                                             61
      IND(IFK)=IND(I)                                                         62
      IND(I)=IV                                                               63
      GO TO 80                                                                64
   70 CONTINUE                                                                65
   80 CONTINUE                                                                66
      I=IF+1                                                                  67
   90 IP=I                                                                    68
      RETURN                                                                  69
      END                                                                     70
      SUBROUTINE KB04YA (IS,IF,IP,IM,NALM,COUNT,M,MARK,N,IND)                 71
      DIMENSION COUNT(1),MARK(1),IND(1)                                       72
      IF(IP-IF)40,40,10                                                       73
C       NO SEGMENTATION                                                       74
   10 IY=(IS+IF)/2                                                            75
      DO 20 I=IS,IF                                                           76
      II=I                                                                    77
      IF(COUNT(I)-COUNT(IY))30,20,30                                          78
   20 CONTINUE                                                                79
C      ALL EQUAL INCREASE IM                                                  80
      MARK(IM)=-MARK(IM)                                                      81
      GO TO 90                                                                82
C      INTERCHANGE                                                            83
   30 AV=COUNT(II)                                                            84
      COUNT(II)=COUNT(IY)                                                     85
      COUNT(IY)=AV                                                            86
      IV=IND(II)                                                              87
      IND(II)=IND(IY)                                                         88
      IND(IY)=IV                                                              89
      GO TO 90                                                                90
   40 NP1=N+1                                                                 91
      M3=MARK(IM)/NP1                                                         92
      MARK(NALM)=M3*NP1+IP                                                    93
      MARK(IM)=NALM*NP1+IS                                                    94
      IF(IP-IS-M)50,50,60                                                     95
   50 MARK(IM)=-MARK(IM)                                                      96
   60 IF(IF+1-IP-M)70,70,80                                                   97
   70 MARK(NALM)=-MARK(NALM)                                                  98
   80 NALM=NALM+1                                                             99
   90 RETURN                                                                 100
      END                                                                    101
      SUBROUTINE KB04ZA(COUNT,N,IS,IND)                                      102
      DIMENSION COUNT(1),IND(1)                                              103
      M=N                                                                    104
   10 M=M/2                                                                  105
      IF(M)90,90,20                                                          106
   20 K=N-M                                                                  107
      J=IS                                                                   108
   30 I=J                                                                    109
   40 IPM=I+M                                                                110
      IF (COUNT(I)-COUNT(IPM)) 60,50,80                                      111
   50 IF (IND(I)-IND(IPM)) 80,80,70                                          112
   60 AV=COUNT(I)                                                            113
      COUNT(I)=COUNT(IPM)                                                    114
      COUNT(IPM)=AV                                                          115
   70 IV=IND(I)                                                              116
      IND(I)=IND(IPM)                                                        117
      IND(IPM)=IV                                                            118
      I=I-M                                                                  119
      IF(I-1)80,40,40                                                        120
   80 J=J+1                                                                  121
      IF (J-IS+1-K) 30,30,10                                                 122
   90 RETURN                                                                 123
      END                                                                    124
